#include <stdint.h>
#include <stdbool.h>
#include <FXRTOS.h>
#include <MDR32F9Qx_rst_clk.h>
#include <MDR32F9Qx_port.h>

#include "canfestival.h"
#include "timerscfg.h"
#include "rpdo_007.h"
#include "io_control.h"

enum {
    CAN_THREAD_STACK_SZ = 512,
    CAN_BAUDRATE = 250000UL
};

static fx_canopen_t fx_can_1;

extern UNS8 rpdo_007_bDeviceNodeId;
static RST_CLK_FreqTypeDef rst_clocks;

//
// LED-blinking thread.
//
void static
blink_loop(void* arg)
{
	for (;;)
	{
		MDR_PORTB->RXTX |= 4;
		fx_thread_sleep(100);
		MDR_PORTB->RXTX &= ~4;
		fx_thread_sleep(100);
		MDR_PORTB->RXTX |= 4;
		fx_thread_sleep(100);
		MDR_PORTB->RXTX &= ~4;
		fx_thread_sleep(1000);
	}
}

//!
//! This function is called by the kernel as a result of any interrupt.
//! HAL or CPU interfaces should be used in order to distinct one vector
//! from another. This demo uses HAL function to obtain the vector.
//!
//! TODO: Pass proper CAN and timer handlers to ISRs
//!
void
fx_intr_handler(void)
{
	switch (hal_intr_get_current_vect())
	{
	case CAN1_IRQn:
		can_recv_isr(&fx_can_1);
		break;
 
    case CAN2_IRQn:
        break;


	default:
		break;
	}
}

//
// Application definition.
//
void
fx_app_init(void)
{
	static fx_thread_t t0;
	static uint32_t stk0[512 / sizeof(uint32_t)];
	fx_thread_init(&t0, blink_loop, NULL, 12, (void*)stk0, sizeof(stk0), false);

    NVIC_SetPriority(TIMER1_IRQn, 4);
	NVIC_SetPriority(CAN1_IRQn, 4);
	NVIC_EnableIRQ(CAN1_IRQn);
	NVIC_EnableIRQ(TIMER1_IRQn);

    static uint32_t timer_stk[512 / sizeof(uint32_t)];
	initTimer(rst_clocks.CPU_CLK_Frequency, timer_stk, sizeof(timer_stk), 3);
    
    static uint32_t can_stk[(CAN_THREAD_STACK_SZ) / sizeof(uint32_t)];
	fx_can_1.co_data = &rpdo_007_Data;
	fx_can_1.thread_stack = can_stk;
	fx_can_1.thread_stack_size = sizeof(can_stk);
	fx_can_1.thread_priority = 3;
	fx_can_1.hw_can = MDR_CAN1;
	fx_can_1.baudrate = CAN_BAUDRATE;
	fx_can_1.hclk = rst_clocks.CPU_CLK_Frequency;
	canInit(&fx_can_1);
	setNodeId(fx_can_1.co_data, rpdo_007_bDeviceNodeId);
	setState(fx_can_1.co_data, Initialisation);
	setState(fx_can_1.co_data, Operational);
}

void static
clk_config(void)
{
	RST_CLK_DeInit();

	RST_CLK_HSEconfig(RST_CLK_HSE_ON);
	if (RST_CLK_HSEstatus() == ERROR)
    {
        for(;;);
    }

	RST_CLK_CPU_PLLconfig(RST_CLK_CPU_PLLsrcHSEdiv1, RST_CLK_CPU_PLLmul5); //RST_CLK_CPU_PLLmul5);

	RST_CLK_CPU_PLLcmd(ENABLE);
	if (RST_CLK_CPU_PLLstatus() == ERROR)
    {
        for (;;);
    }

	RST_CLK_CPU_PLLuse(ENABLE);
	RST_CLK_CPUclkSelection(RST_CLK_CPUclkCPU_C3);
}

int main(void)
{
	clk_config();
    
	led_init();
	led_write(0);
	button_init();
    
	//
	// Start system tick source.
	//
	RST_CLK_GetClocksFreq(&rst_clocks);
	SysTick->LOAD  = (rst_clocks.CPU_CLK_Frequency / 1000) - 1;
	SysTick->VAL   = 0;
	SysTick->CTRL  = (1 << 2) | (1 << 0) | (1 << 1);

	//
	// Enable the peripheral clock
	//
	//MDR_RST_CLK->PER_CLOCK = 0xFFFFFFFF;

	// PORT CAN1
	RST_CLK_PCLKcmd(RST_CLK_PCLK_PORTA, ENABLE);
	MDR_PORTA->OE     &= ~(PORT_Pin_6 | PORT_Pin_7);
	MDR_PORTA->FUNC   &= ~(PORT_FUNC_MODE6_Msk | PORT_FUNC_MODE7_Msk);
	MDR_PORTA->FUNC   |= ((0x02 << PORT_FUNC_MODE6_Pos) | (0x02 << PORT_FUNC_MODE7_Pos));
	MDR_PORTA->ANALOG |= PORT_Pin_6 | PORT_Pin_7;
	MDR_PORTA->PULL   &= ~((PORT_Pin_6 | PORT_Pin_7) << PORT_PULL_DOWN_Pos);
	MDR_PORTA->PULL   |= ((PORT_Pin_6 | PORT_Pin_7) << PORT_PULL_UP_Pos);
	MDR_PORTA->PWR    &= ~(PORT_PWR6_Msk | PORT_PWR7_Msk);
	MDR_PORTA->PWR    |= ((0x02 << PORT_PWR6_Pos) | (0x02 << PORT_PWR7_Pos));
	MDR_PORTA->GFEN   |= (PORT_Pin_6 | PORT_Pin_7);

	//
	// Set the lowest possible priority for PendSV interrupt.
	//
	NVIC_SetPriority(PendSV_IRQn, 0xFF);

	//
	// Kernel start. This function must be called with interrupts disabled.
	//
	fx_kernel_entry();
}

void system_stop(void)
{
	while (1);
}
