// Includes for the Canfestival driver
#include <stm32f1xx_hal.h>
#include <stm32f1xx_ll_tim.h>

#include <cm3/canfestival.h>
#include <timer.h>
#include <can.h>
#include "stm32_defs.h"

static TIM_HandleTypeDef TIM_TimeBaseStructure;

TIMEVAL last_counter_val = 0;
TIMEVAL elapsed_time = 0;


void clearTimer(void)
{
	//NVIC_InitTypeDef NVIC_InitStructure;

	HAL_NVIC_DisableIRQ(TIM3_IRQn);
	//NVIC_InitStructure.NVIC_IRQChannel = TIM3_IRQn;
	//NVIC_InitStructure.NVIC_IRQChannelPreemptionPriority = 0;
	//NVIC_InitStructure.NVIC_IRQChannelCmd = DISABLE;
	//NVIC_Init(&NVIC_InitStructure);

	/* TIM3 disable counter */
	HAL_TIM_Base_Stop_IT(&TIM_TimeBaseStructure);
	//TIM_Cmd(TIM3, DISABLE);

	/* TIM Interrupts disable */
	//TIM_ITConfig(TIM3, TIM_IT_Update, DISABLE);

	__HAL_RCC_TIM3_CLK_DISABLE();
	//RCC_APB1PeriphClockCmd(RCC_APB1Periph_TIM3, DISABLE);

}

void start_callback(CO_Data* d, UNS32 id)
{
}

// Initializes the timer, turn on the interrupt and put the interrupt time to zero
void initTimer(void)
{
	//NVIC_InitTypeDef NVIC_InitStructure;

	/* TIM3 clock enable */
	__HAL_RCC_TIM3_CLK_ENABLE();
	//RCC_APB1PeriphClockCmd(RCC_APB1Periph_TIM3, ENABLE);

	/* Enable the TIM3 gloabal Interrupt */
	HAL_NVIC_EnableIRQ(TIM3_IRQn);
	//NVIC_InitStructure.NVIC_IRQChannel = TIM3_IRQn;
	HAL_NVIC_SetPriority(TIM3_IRQn, 0, 0);
	//NVIC_InitStructure.NVIC_IRQChannelPreemptionPriority = 0;
	//NVIC_InitStructure.NVIC_IRQChannelCmd = ENABLE;
	//NVIC_Init(&NVIC_InitStructure);

	/* Compute the prescaler value */
	/* SystemCoreClock variable holds HCLK frequency and is defined in system_stm32f10x.c file */
	uint16_t PrescalerValue = (uint16_t)(SystemCoreClock  / 100000) - 1;

	/* Time base configuration */
	TIM_TimeBaseStructure.Instance = TIM3;
	TIM_TimeBaseStructure.Init.Period = 65535;
	TIM_TimeBaseStructure.Init.Prescaler = PrescalerValue;
	TIM_TimeBaseStructure.Init.ClockDivision = 0;
	TIM_TimeBaseStructure.Init.CounterMode = TIM_COUNTERMODE_UP;
	TIM_TimeBaseStructure.Init.RepetitionCounter = 0;
	////!!! TimHandle.Init.AutoReloadPreload = TIM_AUTORELOAD_PRELOAD_DISABLE;

	HAL_TIM_Base_Init(&TIM_TimeBaseStructure);

	//__HAL_TIM_CLEAR_IT(&TIM_TimeBaseStructure);
	//TIM_ClearITPendingBit(TIM3, TIM_SR_UIF);

	/* TIM3 disable counter */
	//TIM_Cmd(TIM3, DISABLE);

	/* Preset counter for a safe start */
	//__HAL_TIM_SET_COUNTER(&TIM_TimeBaseStructure, 0);
	//TIM_SetCounter(TIM3, 0);

	/* TIM Interrupts enable */
	//__HAL_TIM_ENABLE_IT(&TIM_TimeBaseStructure, TIM_IT_UPDATE);
	//TIM_ITConfig(TIM3, TIM_IT_Update, ENABLE);

	// this is needed for correct canfestival virtual timer management start
	SetAlarm(NULL, 0, start_callback, 0, 0);
}

//Set the timer for the next alarm.
void setTimer(TIMEVAL value)
{
	uint32_t timer = LL_TIM_GetCounter(TIM_TimeBaseStructure.Instance);
	//uint32_t timer = TIM_GetCounter(TIM3);        // Copy the value of the running timer
	elapsed_time += timer - last_counter_val;
	last_counter_val = 65535 - value;
	LL_TIM_SetCounter(TIM_TimeBaseStructure.Instance, 65535 - value);
	//TIM_SetCounter(TIM3, 65535-value);

	HAL_TIM_Base_Start_IT(&TIM_TimeBaseStructure);
	//TIM_Cmd(TIM3, ENABLE);
}

//Return the elapsed time to tell the Stack how much time is spent since last call.
TIMEVAL getElapsedTime(void)
{
	uint32_t timer = LL_TIM_GetCounter(TIM_TimeBaseStructure.Instance);
	//uint32_t timer = TIM_GetCounter(TIM3);        // Copy the value of the running timer
	if (timer < last_counter_val)
		timer += 65535;
	TIMEVAL elapsed = timer - last_counter_val + elapsed_time;
	//printf("elapsed %lu - %lu %lu %lu\r\n", elapsed, timer, last_counter_val, elapsed_time);
	return elapsed;
}

// This function handles Timer 3 interrupt request.
void TIM3_IRQHandler(void)
{
	//printf("--\r\n");
	//if(TIM_GetFlagStatus(TIM3, TIM_SR_UIF) == RESET)
	if (0 == LL_TIM_IsActiveFlag_UPDATE(TIM_TimeBaseStructure.Instance))
		return;
	last_counter_val = 0;
	elapsed_time = 0;
	__HAL_TIM_CLEAR_IT(&TIM_TimeBaseStructure, TIM_IT_UPDATE);
	//TIM_ClearITPendingBit(TIM3, TIM_SR_UIF);
	TimeDispatch();
}
