#ifndef __CANFESTIVAL_PORTS_FXRTOS_H__
#define __CANFESTIVAL_PORTS_FXRTOS_H__

#include "data.h"
#include <FXRTOS.h>


//
// @brief Send a CAN message
// @param port CanFestival file descriptor
// @param *m The CAN message to send
// @return 0 if succes
//
extern UNS8 canSend(CAN_PORT notused, Message* msg);


//
// @brief Change the CANOpen device baudrate
// @param port CanFestival file descriptor
// @param *baud The new baudrate to assign
// @return
//       - 0 is returned upon success or if not supported by the CAN driver.
//       - errorcode from the CAN driver is returned if an error occurs. (if implemented in the CAN driver)
//
extern UNS8 canChangeBaudRate(CAN_PORT port, char* baud);


//
// Size buffer for CAN messages
//
#define COUNT_BUF_CAN_MSG          (1 << 5)
#define MASK_BUF_CAN_MSG           (COUNT_BUF_CAN_MSG - 1)


//
// CANopen
//
typedef struct _fx_canopen_t
{
	CO_Data*         co_data;

	fx_mutex_t       mtx_codata;

	//fx_isr_t         isr_rx;
	fx_dpc_t         dpc_rx;
	fx_sem_t         sem_rx;
	fx_mutex_t       mtx_tx;
	fx_thread_t      thread_rx;

	uint32_t*        thread_stack;
	uint32_t         thread_stack_size;
	unsigned int     thread_priority;

	Message          buf_can_rx_msg[COUNT_BUF_CAN_MSG];
	uint32_t         iw_buf_can_msg;
	uint32_t         ir_buf_can_msg;

	void*            hw_can;
	uint32_t         baudrate;
	uint32_t         hclk;
}
fx_canopen_t;


#define fx_COData_as_mtx(co_data)           (&(((fx_canopen_t*)((co_data)->canHandle))->mtx_codata))

//
// Initializes the CAN
//
uint32_t canInit(fx_canopen_t* can_open);

void timer_event_isr(void* arg);
void can_recv_isr(void* arg);

#endif
