/* ----------------------------------------------------------------------------
 *         SAM Software Package License
 * ----------------------------------------------------------------------------
 * Copyright (c) 2016, Atmel Corporation
 *
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 * - Redistributions of source code must retain the above copyright notice,
 * this list of conditions and the disclaimer below.
 *
 * Atmel's name may not be used to endorse or promote products derived from
 * this software without specific prior written permission.
 *
 * DISCLAIMER: THIS SOFTWARE IS PROVIDED BY ATMEL "AS IS" AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NON-INFRINGEMENT ARE
 * DISCLAIMED. IN NO EVENT SHALL ATMEL BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA,
 * OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * ----------------------------------------------------------------------------
 */

/*----------------------------------------------------------------------------
 *        Headers
 *----------------------------------------------------------------------------*/

#include <sdmmc.h>
#include <system.h>
#include "board_support.h"

bool board_get_hsmci_card_detect_status(uint32_t periph_id)
{
	/* no detection, assume card is always present */
	return true;
}

bool board_set_hsmci_card_power(uint32_t periph_id, bool on)
{
	/* assume card power is always ON */
	return true;
}

bool board_init(const char* cfg, struct _hsmci_set* set)
{
	char cfg_path[256];
    size_t path_len = strlen(cfg) + strlen("base") + 1;    
    strncpy(cfg_path, cfg, path_len);
    strcat(cfg_path, "base");
	uintptr_t pbase = 0;

    if (fx_reg_get_value(cfg_path, FX_REG_TYPE_DWORD, &pbase, sizeof(pbase)))
    {
        DPRINTF(DBG_ERROR, "Device base address not set");
        return false;
    }
	set->regs = (void*)fx_dev_map(pbase, sizeof(Hsmci));
	if (set->regs == NULL)
	{
		return false;
	}

    path_len = strlen(cfg) + strlen("periph_id") + 1;
    strncpy(cfg_path, cfg, path_len);
    strcat(cfg_path, "periph_id");

    if (fx_reg_get_value(cfg_path, FX_REG_TYPE_DWORD, &set->id, sizeof(set->id)))
	{
		DPRINTF(DBG_ERROR, "Device peripheral id is not set");
		return false;
	}

	path_len = strlen(cfg) + strlen("dmac_base") + 1;
	strncpy(cfg_path, cfg, path_len);
    strcat(cfg_path, "dmac_base");
	if (fx_reg_get_value(cfg_path, FX_REG_TYPE_DWORD, &pbase, sizeof(pbase)))
    {
        DPRINTF(DBG_ERROR, "DMAC dmac_base address is not set. Continue without DMA");
    }
	else
	{
		uintptr_t misalign = pbase % 0x1000;
		set->dmac = fx_dev_map(pbase - misalign, 0x2000) + misalign;
		if (set->dmac == (void*)misalign)
		{
			DPRINTF(DBG_WARNING, "Dmac address is not set. Continue without DMA");
			set->use_dma = false;
		}
	}

	set->ops.get_card_detect_status = board_get_hsmci_card_detect_status;
	set->ops.set_card_power = board_set_hsmci_card_power;

	return true;
}
