/**
****************************************************************************************************
*  @file   sdmmc.h
*  @brief  SD/MMC controller and device driver.
*
****************************************************************************************************
*  Copyright (C) 2019-2020 Eremex
*  All rights reserved.
***************************************************************************************************/

#ifndef _SDMMC_HEADER_
#define _SDMMC_HEADER_

#include <system.h>

#include <hsmci.h>

#include "sdmmc_cmd.h"
#include "sdmmc_api.h"

#define BLOCK_CNT_MAX               256u

struct _hsmci_ops {
	bool (*get_card_detect_status)(uint32_t periph_id);
	bool (*set_card_power)(uint32_t periph_id, bool power);
};

/* This structure is private to the HSMCI Driver.
 * Allocate it but ignore its members. */
struct _hsmci_set {
	uint32_t id;                  /* HSMCI peripheral ID (ID_HSMCIx) */
	//struct _pin wp_pin;           /* HSMCI write protection pin */
	Hsmci *regs;                  /* set of HSMCI hardware registers */
	void *dmac;				  /* DMA controller registers */
	uint32_t dma_rx_ch_num;			  /* Transmit DMA channel number */
	uint32_t dma_tx_ch_num;			  /* Receive DMA channel number */
	struct _hsmci_ops ops;
	//struct _dma_channel *dma_tx_channel;
	//struct _dma_channel *dma_rx_channel;
	uint32_t nxt_evts;            /* HSMCI IRQs we're still waiting for */
	uint32_t dwXfrNdx;            /* DMA transferred data index (bytes) */
	uint32_t dwXSize;             /* DMA transfer size (bytes) */
	bool use_irq;             	  /* IRQ or polling mode */
	bool use_dma;             	  /* use dma transfer */
	uint32_t dev_freq;            /* frequency of clock provided to memory
	                               * device, in Hz */
	uint8_t tim_mode;             /* timing mode aka bus speed mode */
	volatile uint8_t state;
	struct _SdmmcCommand *cmd;    /* pointer to the command being processed */
	uint16_t blk_index;           /* count of data blocks tranferred already,
	                               * in the context of the command and data
	                               * transfer being executed */
	uint8_t resp_len;             /* size of the response, once retrieved,
	                               * in the context of the command being
	                               * executed, expressed in 32-bit words */
	mutex_t dma_unlocks_mutex;
};

typedef struct sdmmc_s {
    struct _hsmci_set settings;
    sSdCard card;
    uint8_t buf[BLOCK_CNT_MAX * 512ul];
} sdmmc_t;

#endif
