/**
****************************************************************************************************
*  @file   system.c
*  @brief  OS and compiler dependencies for driver.
*
****************************************************************************************************
*  Copyright (C) 2019-2020 Eremex
*  All rights reserved.
***************************************************************************************************/

#include "system.h"

void
timer_start_timeout(struct _timeout *timeout, uint32_t count)
{
    timeout->start = fx_timer_get_tick_count();
    timeout->count = count;
}

bool 
timer_timeout_reached(struct _timeout* timeout)
{
    uint32_t now = fx_timer_get_tick_count();

    if (now > (timeout->start + timeout->count))
        return true;
    else
        return false;
}

void
sleep_ms(uint32_t msec)
{
    fx_thread_sleep(msec);
}

void
sleep_us(uint32_t usec)
{
    uint32_t count;
    if (usec > 0 && usec < TICK_PERIOD)
    {
        count = 1;
    }
    else
    {
        count = usec/TICK_PERIOD;
    }
    fx_thread_sleep(count);
}

bool mutex_try_lock(mutex_t* mutex)
{
    if (fx_mutex_timedacquire(mutex, 0))
        return false;
    else
        return true;
}

void mutex_lock(mutex_t* mutex)
{
    fx_mutex_timedacquire(mutex, FX_THREAD_INFINITE_TIMEOUT);
}

void mutex_unlock(mutex_t* mutex)
{
    fx_mutex_release(mutex);
}

/*
bool mutex_is_locked(const mutex_t* mutex)
{
    if (fx_mutex_get_owner(mutex) != NULL)
        return true;
    else
        return false;
}
*/

bool irq_add_handler(uint32_t periph_id, void(*isr)(void*), void* isr_arg)
{
    return false;
}

bool irq_enable(uint32_t periph_id)
{
    return false;
}
