/*
 */

#include <string.h>

#include "lwip/tcpip.h"
#include "lwip/apps/mqtt.h"
#include "arch/cc.h"

#include <stm32f4_discovery.h>

extern err_t ethernetif_init(struct netif *netif);

static void network_setup( void *pvArgument )
{
	static struct netif net_intrface;
	ip_addr_t my_ipaddr, my_netmask, my_gateway;

	ip_addr_set_zero( &my_gateway );
	ip_addr_set_zero( &my_ipaddr );
	ip_addr_set_zero( &my_netmask );

	IP4_ADDR((&my_gateway), 192,168,128,1);
	IP4_ADDR((&my_ipaddr), 192,168,128,11);
	IP4_ADDR((&my_netmask), 255,255,255,0);

	netif_add(
			&net_intrface, &my_ipaddr, &my_netmask, &my_gateway, (void*)2, ethernetif_init, tcpip_input
	);

	netif_set_default( &net_intrface );
	netif_set_up( &net_intrface );
}

static uint16_t led_id;
static mqtt_client_t* mqtt_client;

static const struct mqtt_connect_client_info_t mqtt_client_info =
{
	"fx_rtos_iot",
	NULL, /* user */
	NULL, /* pass */
	100,  /* keep alive */
	NULL, /* will_topic */
	NULL, /* will_msg */
	0,    /* will_qos */
	0     /* will_retain */
#if LWIP_ALTCP && LWIP_ALTCP_TLS
	, NULL
#endif
};

static void
mqtt_inc_publish_cb(void* arg, const char* topic, u32_t tot_len)
{
	uint16_t* pin = arg;
	*pin = LED6_PIN;
}

static void
mqtt_inc_data_cb(void* arg, const u8_t* data, u16_t len, u8_t flags)
{
	GPIO_PinState state;
	if (strncmp((char*)data, "on", 2))
	{
		state = GPIO_PIN_RESET;
	}
	else if (strncmp((char*)data, "off", 3))
	{
		state = GPIO_PIN_SET;
	}
	uint16_t* pin = arg;
	HAL_GPIO_WritePin(LED6_GPIO_PORT, *pin, state);
}

static void
mqtt_connection_cb(mqtt_client_t *client, void *arg, mqtt_connection_status_t status)
{
	LWIP_UNUSED_ARG(arg);
	LWIP_UNUSED_ARG(client);

	if (status == MQTT_CONNECT_ACCEPTED) {
		mqtt_publish(mqtt_client, "hello/world", "Blablabla!", 10, 0, 0, NULL, NULL);
	    mqtt_set_inpub_callback(mqtt_client, mqtt_inc_publish_cb, mqtt_inc_data_cb, &led_id);
	    mqtt_subscribe(mqtt_client, "leds/blue", 0, NULL, NULL);
	}
}

void
mqtt_example_init(void)
{
    tcpip_init( network_setup, NULL );
    ip_addr_t broker_ipaddr;
    IP4_ADDR((&broker_ipaddr), 192,168,128,10);

    mqtt_client = mqtt_client_new();

    mqtt_client_connect(mqtt_client,
          &broker_ipaddr, MQTT_PORT,
          mqtt_connection_cb, LWIP_CONST_CAST(void*, &mqtt_client_info),
          &mqtt_client_info);
}
