/*
 * FX-RTOS networking demo.
 */

#include "FXRTOS.h"
#include "lpc17xx.h"
#include "lwip/tcpip.h"
#include "apps/fsdata.h" // for lwip2 use "lwip/apps/fs.h"

void led_init(void);
void led_on(int);
void led_off(int);
err_t drv_eth_init( struct netif* );
void drv_eth_interrupt(void);

static const unsigned char indexdata[] =
"HTTP/1.1 200 OK\r\n\
Content-type: text/html\r\n\
\r\n\
<html> \
<head><title>Demo page</title></head> \
<body> \
FX-RTOS network demo page. \
</body> \
</html>";

const struct fsdata_file file__index_shtml[] = { {
    NULL,
    "/index.shtml",
    indexdata,
    sizeof(indexdata),
    1,
}};

const struct fsdata_file* const httpd_fs_root = file__index_shtml;

static void network_setup( void *pvArgument )
{
    static struct netif net_intrface;
    ip_addr_t my_ipaddr, my_netmask, my_gateway;

    ip_addr_set_zero( &my_gateway );
    ip_addr_set_zero( &my_ipaddr );
    ip_addr_set_zero( &my_netmask );

    IP4_ADDR((&my_gateway), 192,168,128,1);
    IP4_ADDR((&my_ipaddr), 192,168,128,11);
    IP4_ADDR((&my_netmask), 255,255,255,0);

    netif_add(
        &net_intrface, &my_ipaddr, &my_netmask, &my_gateway, (void*)2, drv_eth_init, tcpip_input 
    );

    netif_set_default( &net_intrface );
    netif_set_up( &net_intrface );

    httpd_init();
}

//
// LED-blinking thread 1.
//
static void thread_1(void* arg)
{
    fx_thread_sleep(200);

    tcpip_init( network_setup, NULL );

    while(1)
    {
        led_on(0);
        fx_thread_sleep(500);
        led_off(0);
        fx_thread_sleep(500);
    }
}

//
// LED-blinking thread 2.
//
static void thread_2(void* arg)
{
    while(1)
    {
        led_on(1);
        fx_thread_sleep(250);
        led_off(1);
        fx_thread_sleep(250);
    }
}

//
// LED-blinking thread 3.
//
static void thread_3(void* arg)
{
    while(1)
    {
        led_on(2);
        fx_thread_sleep(125);
        led_off(2);
        fx_thread_sleep(125);
    }
}

void fx_app_init(void)
{
    static fx_thread_t t1,t2,t3; 
    hw_cpu_intr_enable();

    fx_thread_init(&t1, thread_1, NULL, 10, (void*) 0x10007000, 0x400, false);
    fx_thread_init(&t2, thread_2, NULL, 11, (void*) 0x10006C00, 0x400, false);
    fx_thread_init(&t3, thread_3, NULL, 12, (void*) 0x10006800, 0x400, false);
}

//
// Application entry point.
//
int main(void)
{
    //
    // Initialize clocks and PLL.
    //
    SystemInit();
    SystemCoreClockUpdate();

    //
    // Initialize LED hardware.
    //
    led_init(); 

    //
    // Set PendSV to lowest possible priority level.
    //
    NVIC_SetPriority(PendSV_IRQn, 0xFF);
    NVIC_SetPriority(SysTick_IRQn, 0xFF);
    NVIC_SetPriorityGrouping(2);
    NVIC_EnableIRQ(ENET_IRQn);
    
    //
    // Start system tick source.
    //
    SysTick->LOAD  = 100000 - 1;
    SysTick->VAL   = 0;
    SysTick->CTRL  = (1 << 2) | (1 << 0) | (1 << 1);

    //
    // Start scheduling loop.
    //
    fx_kernel_entry();
        
    return 0;
}

void fx_intr_handler(void)
{
    unsigned int vector = hal_intr_get_current_vect();

    //
    // Check whether the vector is ours...
    //
    switch (vector)
    {
    case  ENET_IRQn:
    
        drv_eth_interrupt();
        break;

    default: break;
    }
}

void system_stop(int* regs)
{ 
    while(1); 
}
