/** 
  **************************************************************************************************
  *  @file   lpc17xx_eth.h
  *  @brief  Interface of Ethernet driver for LPC17xx boards.
  **************************************************************************************************
  *  Copyright (C) 2020 Eremex
  *  All rights reserved.
  *************************************************************************************************/

#include <FXRTOS.h>

//----------------------------------------------------------------------------------------------------

typedef struct _lpc17xx_eth_descr_t
{
  uint32_t buffer;
  uint32_t control;
}
lpc17xx_eth_descr_t;

//----------------------------------------------------------------------------------------------------

typedef struct _lpc17xx_rx_status_t
{
  uint32_t status;
  uint32_t status_hash_crc;
}
lpc17xx_rx_status_t;

//----------------------------------------------------------------------------------------------------

typedef struct _lpc17xx_eth_dev_t
{
  fx_dpc_t dpc;

  size_t max_frame_len;

  uint8_t* mem_area;
  size_t mem_sz;

  lpc17xx_eth_descr_t* tx_descriptors;
  uint32_t* tx_status;
  size_t tx_descriptors_num;

  lpc17xx_eth_descr_t* rx_descriptors;
  lpc17xx_rx_status_t* rx_status;
  size_t rx_descriptors_num;

  void (*rx_signal)(void* rx_arg);
  void* rx_arg;
  unsigned rx_service_req;

  uint8_t mac_addr[6];

  int tx_consumed;
}
lpc17xx_eth_dev_t;

//----------------------------------------------------------------------------------------------------

typedef struct _lpc17xx_eth_params_t
{
  bool (*phy_init)(bool* full_duplex, bool* full_speed);
  size_t tx_descr_num;
  size_t rx_descr_num;
  size_t max_frame_len;
  uintptr_t mem; 
  size_t mem_sz;
  uint8_t mac_addr[6];
  bool zero_copy;
}
lpc17xx_eth_params_t;

//----------------------------------------------------------------------------------------------------

bool lpc17xx_eth_init(lpc17xx_eth_dev_t* dev, void (*rx_signal)(void*), void* rx_arg, lpc17xx_eth_params_t* info);
void lpc17xx_eth_deinit(lpc17xx_eth_dev_t* dev);

bool lpc17xx_eth_set_mac_addr(lpc17xx_eth_dev_t* dev, const uint8_t* mac);
bool lpc17xx_eth_get_mac_addr(lpc17xx_eth_dev_t* dev, uint8_t* mac);

//----------------------------------------------------------------------------------------------------

bool lpc17xx_eth_acquire_tx(lpc17xx_eth_dev_t* dev, void** frame, size_t* sz);
bool lpc17xx_eth_send(lpc17xx_eth_dev_t* dev, void* frame, size_t sz);
bool lpc17xx_eth_receive(lpc17xx_eth_dev_t* dev, void** frame, size_t* sz);
bool lpc17xx_eth_release_rx(lpc17xx_eth_dev_t* dev, void* frame, size_t sz);

void lpc17xx_eth_isr_handler(lpc17xx_eth_dev_t* dev);
