/** 
  ******************************************************************************
  *  @file   sys_arch.h
  *  @brief  OS porting layer for lwIP.
  ******************************************************************************
  *  Copyright (C) JSC EREMEX, 2008-2020.
  *  Redistribution and use in source and binary forms, with or without 
  *  modification, are permitted provided that the following conditions are met:
  *  1. Redistributions of source code must retain the above copyright notice,
  *     this list of conditions and the following disclaimer.
  *  2. Redistributions in binary form must reproduce the above copyright 
  *     notice, this list of conditions and the following disclaimer in the 
  *     documentation and/or other materials provided with the distribution.
  *  3. Neither the name of the copyright holder nor the names of its 
  *     contributors may be used to endorse or promote products derived from 
  *     this software without specific prior written permission.
  *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
  *  AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE 
  *  IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
  *  ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE 
  *  LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR 
  *  CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF 
  *  SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
  *  INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN 
  *  CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
  *  ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
  *  POSSIBILITY OF SUCH DAMAGE.
  *****************************************************************************/

#ifndef __ARCH_SYS_ARCH_H__
#define __ARCH_SYS_ARCH_H__

#include <FXRTOS.h>
#undef s_addr

#define SYS_MBOX_NULL ( ( fx_msgq_t* ) NULL )
#define SYS_SEM_NULL ( ( fx_sem_t* ) NULL )
#define SYS_DEFAULT_THREAD_STACK_DEPTH	2000

typedef struct sys_sem_t
{
    fx_sem_t sema;
    bool valid;
}
sys_sem_t;

typedef struct sys_mbox_t
{
    fx_msgq_t msgq;
    bool valid;
}
sys_mbox_t;

typedef fx_mutex_t sys_mutex_t;
typedef fx_thread_t* sys_thread_t;

#ifndef CFG_FXRTOS_SECURE
typedef spl_t sys_prot_t;
#else
typedef int sys_prot_t;
#endif

#define sys_mbox_valid( x ) ( (x)->valid )
#define sys_mbox_set_invalid( x ) ( (x)->valid = false )
#define sys_sem_valid( x )  ( (x)->valid )
#define sys_sem_set_invalid( x ) ( (x)->valid = false )

#endif

