/**
 ******************************************************************************
 * @file      startup_stm32f411retx.s
 * @author    Auto-generated by STM32CubeIDE
 * @brief     STM32F411RETx device vector table for GCC toolchain.
 *            This module performs:
 *                - Set the initial SP
 *                - Set the initial PC == Reset_Handler,
 *                - Set the vector table entries with the exceptions ISR address
 *                - Branches to main in the C library (which eventually
 *                  calls main()).
 ******************************************************************************
 * @attention
 *
 * <h2><center>&copy; Copyright (c) 2020 STMicroelectronics.
 * All rights reserved.</center></h2>
 *
 * This software component is licensed by ST under BSD 3-Clause license,
 * the "License"; You may not use this file except in compliance with the
 * License. You may obtain a copy of the License at:
 *                        opensource.org/licenses/BSD-3-Clause
 *
 ******************************************************************************
 */

.syntax unified
.cpu cortex-m4
.fpu softvfp
.thumb

.global g_pfnVectors
.global Default_Handler

/* start address for the initialization values of the .data section.
defined in linker script */
.word _sidata
/* start address for the .data section. defined in linker script */
.word _sdata
/* end address for the .data section. defined in linker script */
.word _edata
/* start address for the .bss section. defined in linker script */
.word _sbss
/* end address for the .bss section. defined in linker script */
.word _ebss

/**
 * @brief  This is the code that gets called when the processor first
 *          starts execution following a reset event. Only the absolutely
 *          necessary set is performed, after which the application
 *          supplied main() routine is called.
 * @param  None
 * @retval : None
*/

  .section .text.Reset_Handler
  .weak Reset_Handler
  .type Reset_Handler, %function
Reset_Handler:
  ldr   r0, =_estack
  mov   sp, r0          /* set stack pointer */

/* Copy the data segment initializers from flash to SRAM */
  ldr r0, =_sdata
  ldr r1, =_edata
  ldr r2, =_sidata
  movs r3, #0
  b LoopCopyDataInit

CopyDataInit:
  ldr r4, [r2, r3]
  str r4, [r0, r3]
  adds r3, r3, #4

LoopCopyDataInit:
  adds r4, r0, r3
  cmp r4, r1
  bcc CopyDataInit

/* Zero fill the bss segment. */
  ldr r2, =_sbss
  ldr r4, =_ebss
  movs r3, #0
  b LoopFillZerobss

FillZerobss:
  str  r3, [r2]
  adds r2, r2, #4

LoopFillZerobss:
  cmp r2, r4
  bcc FillZerobss

/* Call the clock system intitialization function.*/
  bl  SystemInit
/* Call static constructors */
  bl __libc_init_array
/* Call the application entry point.*/
  bl main

LoopForever:
    b LoopForever

  .size Reset_Handler, .-Reset_Handler

/**
 * @brief  This is the code that gets called when the processor receives an
 *         unexpected interrupt.  This simply enters an infinite loop, preserving
 *         the system state for examination by a debugger.
 *
 * @param  None
 * @retval : None
*/
  .section .text.Default_Handler,"ax",%progbits
Default_Handler:
Infinite_Loop:
  b Infinite_Loop
  .size Default_Handler, .-Default_Handler

/******************************************************************************
*
* The STM32F411RETx vector table.  Note that the proper constructs
* must be placed on this to ensure that it ends up at physical address
* 0x0000.0000.
*
******************************************************************************/
  .section .isr_vector,"a",%progbits
  .type g_pfnVectors, %object
  .size g_pfnVectors, .-g_pfnVectors

g_pfnVectors:
  .word _estack
  .word     Reset_Handler       	    // Reset Handler
  .word     system_stop                 // NMI
  .word     system_stop                 // HardFault
  .word     system_stop                 // MPU Fault Handler
  .word     system_stop                 // Bus Fault Handler
  .word     system_stop                 // Usage Fault Handler
  .word     0                           // Reserved
  .word     0                           // Reserved
  .word     0                           // Reserved
  .word     0                           // Reserved
  .word     system_stop                 // SVCall Handler
  .word     system_stop                 // Debug Monitor Handler
  .word     0                           // Reserved
  .word     hal_swi_entry               // PendSV Handler
  .word     hal_tick_entry              // SysTick Handler
  .word     hal_intr_entry
  .word     hal_intr_entry
  .word     hal_intr_entry
  .word     hal_intr_entry
  .word     hal_intr_entry
  .word     hal_intr_entry
  .word     hal_intr_entry
  .word     hal_intr_entry
  .word     hal_intr_entry
  .word     hal_intr_entry
  .word     hal_intr_entry
  .word     hal_intr_entry
  .word     hal_intr_entry
  .word     hal_intr_entry
  .word     hal_intr_entry
  .word     hal_intr_entry
  .word     hal_intr_entry
  .word     hal_intr_entry
  .word     hal_intr_entry
  .word     hal_intr_entry
  .word     hal_intr_entry
  .word     hal_intr_entry
  .word     hal_intr_entry
  .word     hal_intr_entry
  .word     hal_intr_entry
  .word     hal_intr_entry
  .word     hal_intr_entry
  .word     hal_intr_entry
  .word     hal_intr_entry
  .word     hal_intr_entry
  .word     hal_intr_entry
  .word     hal_intr_entry
  .word     hal_intr_entry
  .word     hal_intr_entry
  .word     hal_intr_entry
  .word     hal_intr_entry
  .word     hal_intr_entry
  .word     hal_intr_entry
  .word     hal_intr_entry
  .word     hal_intr_entry
  .word     hal_intr_entry
  .word     hal_intr_entry
  .word     hal_intr_entry
  .word     hal_intr_entry
  .word     hal_intr_entry
  .word     hal_intr_entry
  .word     hal_intr_entry
  .word     hal_intr_entry
  .word     hal_intr_entry
  .word     hal_intr_entry
  .word     hal_intr_entry
  .word     hal_intr_entry
  .word     hal_intr_entry
  .word     hal_intr_entry
  .word     hal_intr_entry
  .word     hal_intr_entry
  .word     hal_intr_entry
  .word     hal_intr_entry
  .word     hal_intr_entry
  .word     hal_intr_entry
  .word     hal_intr_entry
  .word     hal_intr_entry
  .word     hal_intr_entry
  .word     hal_intr_entry
  .word     hal_intr_entry
  .word     hal_intr_entry
  .word     hal_intr_entry
  .word     hal_intr_entry
  .word     hal_intr_entry
  .word     hal_intr_entry
  .word     hal_intr_entry
  .word     hal_intr_entry
  .word     hal_intr_entry
  .word     hal_intr_entry
  .word     hal_intr_entry
  .word     hal_intr_entry
  .word     hal_intr_entry
  .word     hal_intr_entry
  .word     hal_intr_entry
  .word     hal_intr_entry
  .word     hal_intr_entry
  .word     hal_intr_entry
  .word     hal_intr_entry
