/*
 *  Copyright (C) JSC EREMEX, 2021.
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions are met:
 *  1. Redistributions of source code must retain the above copyright notice,
 *     this list of conditions and the following disclaimer.
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *  3. Neither the name of the copyright holder nor the names of its
 *     contributors may be used to endorse or promote products derived from
 *     this software without specific prior written permission.
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 *  AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 *  IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 *  ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
 *  LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 *  CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 *  SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 *  INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 *  CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 *  ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 *  POSSIBILITY OF SUCH DAMAGE.
 *
 *  NAND memory hardware interface for FTL library.
 */

#ifndef INC_FX_NAND_HARDWARE_H_
#define INC_FX_NAND_HARDWARE_H_

typedef enum
{
	Chip0_Select = 0U,
	Chip0_UnSelect = ~Chip0_Select,
	Chip1_Select = 1U,
	Chip1_UnSelect = ~Chip1_Select
} NAND_Chip;


// Команды для NAND (Micron 32/64/128/256 Gbit)
// Для других производителей и размера памяти команды могут не поддерживаться !!!
// Стандартные команды помечены [Std]

// Reset Operations
#define NAND_CMD_RESET							0xFF	// [Std]
#define NAND_CMD_SYNCHRONOUS_RESET				0xFC
#define NAND_CMD_RESET_LUN						0xFA

// Identification Operations
#define NAND_CMD_READ_ID						0x90	// [Std]
#define NAND_CMD_READ_PARAMETER_PAGE			0xEC
#define NAND_CMD_READ_UNIQUE_ID					0xED

// Configuration Operations
#define NAND_CMD_GET_FEATURES					0xEE
#define NAND_CMD_SET_FEATURES					0xEF

// Status Operations
#define NAND_CMD_READ_STATUS					0x70	// [Std]
#define NAND_CMD_READ_STATUS_ENHANCED			0x78

// Column Address Operations
#define NAND_CMD_CHANGE_READ_COLUMN				0x05
#define NAND_CMD_CHANGE_READ_COLUMN_ENHANCED	0x06
#define NAND_CMD_CHANGE_WRITE_COLUMN			0x85
#define NAND_CMD_CHANGE_ROW_ADDRESS				0x85

// Read Operations
#define NAND_CMD_READ_MODE						0x00
#define NAND_CMD_READ_PAGE_CYCLE1				0x00	// [Std]
#define NAND_CMD_READ_PAGE_CYCLE2				0x30	// [Std]
#define NAND_CMD_READ_PAGE_MULTIPLANE			0x00
#define NAND_CMD_READ_PAGE_CACHE_SEQUENTIAL		0x31
#define NAND_CMD_READ_PAGE_CACHE_RANDOM			0x00
#define NAND_CMD_READ_PAGE_CACHE_LAST			0x3F

// Program Operations
#define NAND_CMD_PROGRAM_PAGE_CYCLE1			0x80	// [Std]
#define NAND_CMD_PROGRAM_PAGE_CYCLE2			0x10	// [Std]
#define NAND_CMD_PROGRAM_PAGE_MULTIPLANE		0x80
#define NAND_CMD_PROGRAM_PAGE_CACHE 			0x80	// [Std]

// Erase Operations
#define NAND_CMD_ERASE_BLOCK_CYCLE1				0x60	// [Std]
#define NAND_CMD_ERASE_BLOCK_CYCLE2				0xD0	// [Std]
#define NAND_CMD_ERASE_BLOCK_MULTIPLANE 		0x60

// Copyback Operations
#define NAND_CMD_COPYBACK_READ 					0x00	// [Std]
#define NAND_CMD_COPYBACK_PROGRAM				0x85	// [Std]
#define NAND_CMD_COPYBACK_PROGRAM_MULTIPLANE	0x85

// Status flags
#define NAND_ERASE_PROGRAM_FAIL                  0x3

void nand_init(void);
void nand_chip_enable(NAND_Chip cs);
bool nand_get_ready(NAND_Chip cs);
void nand_write_cmd(uint8_t cmd);
void nand_write_addr(uint8_t addr);
void nand_write_byte(uint8_t data);
uint8_t nand_read_byte(void);
uint8_t nand_status(NAND_Chip chipselect);

void nand_set_dir_input(void);
void nand_set_dir_output(void);

#endif /* INC_FX_NAND_HARDWARE_H_ */
