/**
  ******************************************************************************
  *  @file   main.c
  *  @brief  FX-RTOS USB demo application.
  *
  ******************************************************************************
  *  Copyright (C) 2020 Eremex
  *  All rights reserved.
  *****************************************************************************/

#include <FXRTOS.h>

#include <MDR32Fx.h>
#include <MDR32F9Qx_rst_clk.h>

extern void device_main(void* arg);

//!
//! Application definition.
//!
void
fx_app_init(void)
{
	static fx_thread_t usbd_thread;

	static uint32_t usbd_stack[ 0x200 / sizeof(uint32_t) ];

	int ret = fx_thread_init(&usbd_thread, device_main, NULL, 10,
	                         (void*)usbd_stack, sizeof(usbd_stack), false);

	(void)ret;
}

//!
//! Exception handler just stop the system and sets LED7.
//!
void
system_stop(void)
{
	while (1);
}

void
Setup_CPU_Clock(void)
{
	//
	// Enable HSE
	//
	RST_CLK_HSEconfig(RST_CLK_HSE_ON);
	if (RST_CLK_HSEstatus() != SUCCESS)
	{
		while (1)
		{
			;
		}
	}
	//
	// CPU_C1_SEL = HSE
	//
	RST_CLK_CPU_PLLconfig(RST_CLK_CPU_PLLsrcHSEdiv1, RST_CLK_CPU_PLLmul10);
	RST_CLK_CPU_PLLcmd(ENABLE);
	if (RST_CLK_CPU_PLLstatus() != SUCCESS)
	{
		while (1)
		{
			;
		}
	}

	//
	// CPU_C3_SEL = CPU_C2_SEL
	//
	RST_CLK_CPUclkPrescaler(RST_CLK_CPUclkDIV1);
	//
	// CPU_C2_SEL = PLL
	//
	RST_CLK_CPU_PLLuse(ENABLE);
	//
	// HCLK_SEL = CPU_C3_SEL
	//
	RST_CLK_CPUclkSelection(RST_CLK_CPUclkCPU_C3);
}

//!
//! Application entry point.
//!
int
main(void)
{
	Setup_CPU_Clock();
	//
	// Start system tick source.
	//
	SysTick->LOAD  = 100000 - 1;
	SysTick->VAL   = 0;
	SysTick->CTRL  = (1 << 2) | (1 << 0) | (1 << 1);

	//
	// Enable the peripheral clock
	//
	MDR_RST_CLK->PER_CLOCK = 0xFFFFFFFF;

	//
	// Set the lowest possible priority for PendSV interrupt.
	//
	NVIC_SetPriority(PendSV_IRQn, 0xFF);

	//
	// Kernel start. This function must be called with interrupts disabled.
	//
	fx_kernel_entry();
	return 0;
}
