/**
 ******************************************************************************
 *  @file   msc_bot.h
 *  @brief  Bulk-only-transfer layer for mass storage.
 *
 ******************************************************************************
LPCUSB, an USB device driver for LPC microcontrollers
Copyright (C) 2006 Bertrik Sikken (bertrik@sikken.nl)
Copyright (c) 2020 Eremex Ltd.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are met:

1. Redistributions of source code must retain the above copyright
   notice, this list of conditions and the following disclaimer.
2. Redistributions in binary form must reproduce the above copyright
   notice, this list of conditions and the following disclaimer in the
   documentation and/or other materials provided with the distribution.
3. The name of the author may not be used to endorse or promote products
   derived from this software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *****************************************************************************/
#ifndef _MSC_BOT_H_
#define _MSC_BOT_H_

#include <stdint.h>

#include "usbdevapi.h"

//!
//! Command block wrapper structure
//!
typedef struct
{
	uint32_t    dwCBWSignature;
	uint32_t    dwCBWTag;
	uint32_t    dwCBWDataTransferLength;
	uint8_t     bmCBWFlags;
	uint8_t     bCBWLun;
	uint8_t     bCBWCBLength;
	uint8_t     CBWCB[16];
} cbw_t;

//!
//! Command status wrapper structure
//!
typedef struct
{
	uint32_t        dwCSWSignature;
	uint32_t        dwCSWTag;
	uint32_t        dwCSWDataResidue;
	uint8_t     bmCSWStatus;
} csw_t;

//!
//! States of BOT state machine
//!
typedef enum
{
	eCBW,
	eDataOut,
	eDataIn,
	eCSW,
	eStalled
} bot_state_t;

typedef struct msc_context_s
{
	uint8_t         in_ep;
	uint8_t         out_ep;
	size_t          transfer_size;  //!< total size of data transfer
	size_t          offset;         //!< offset in current data transfer
	cbw_t           cbw;
	csw_t           csw;
	bot_state_t     state;
	uint8_t*         pbdata;
} msc_context_t;

void msc_reset(msc_context_t* context);
void msc_bulk_out(uint8_t ep, uint8_t ep_status, msc_context_t* context);
void msc_bulk_in(uint8_t ep, uint8_t ep_status, msc_context_t* context);

//!
//! USB Mass storage block device interface
//!
int blockdev_init(void);
int blockdev_write(uint32_t dwAddress, uint8_t* pbuf);
int blockdev_read(uint32_t dwAddress, uint8_t* pbuf);
uint32_t blockdev_get_blk_num(void);
uint32_t blockdev_get_blk_size(void);
int blockdev_get_status(void);

#endif
