
#include <stdarg.h>
#include <unistd.h>

#include "fx_printf.h"

#define EOF -1

int in = 0;
int out = 0;

void
fx_printf(const char *fmt, ...)
{
    static const char digits[] = "0123456789abcdef";
    va_list ap;
    char buf[10];
    char *s;
    unsigned long long u;
    int c, i, pad;
    int is_long = 0;
    int is_long_long = 0;
    unsigned hex_width = 2*sizeof(unsigned);

    va_start(ap, fmt);
    while ((c = *fmt++) != 0) {
        if (c == '%') {
            c = *fmt++;
            if (c == 'l')
            {
                if ('l' == (c = *fmt++))
                {
                    is_long_long = 1;
                    hex_width = 2*sizeof(long long);
                    c = *fmt++;
                }
                else
                {
                    is_long = 1;
                    hex_width = 2*sizeof(long);
                }
            }
            switch (c) {
            case 'c':
                fx_putchar(va_arg(ap, int));
                continue;
            case 's':
                s = va_arg(ap, char *);
                if (s == NULL)
                    s = "<NULL>";
                for (; *s; s++) {
                    fx_putchar((int)*s);
                }
                continue;
            case 'd':
                c = 'u';
            case 'u':
            case 'x':
                if (is_long_long == 1)
                    u = va_arg(ap, unsigned long long);
                else if (is_long == 1)
                    u = va_arg(ap, unsigned long);
                else
                    u = va_arg(ap, unsigned);
                s = buf;
                if (c == 'u') {
                    do
                        *s++ = digits[u % 10U];
                    while (u /= 10U);
                } else {
                    pad = 0;
                    for (i = 0; i < hex_width; i++) {
                        if (pad)
                            *s++ = '0';
                        else {
                            *s++ = digits[u % 16U];
                            if ((u /= 16U) == 0)
                                pad = 1;
                        }
                    }
                }
                while (--s >= buf)
                    fx_putchar((int)*s);
                continue;
            }
        }
        if (c == '\n')
            fx_putchar('\r');
        fx_putchar((int)c);
    }
    va_end(ap);
}

int
fx_puts(const char *s)
{
    fx_printf("%s\n", s);
    return 0;
}
